@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21765 $
- * $Author: hfriederich $
- * $Date: 2008-12-10 07:59:41 +0000 (Wed, 10 Dec 2008) $
+ * $Revision: 22097 $
+ * $Author: rjongbloed $
+ * $Date: 2009-02-21 04:55:10 +0000 (Sat, 21 Feb 2009) $
  */
 
 #ifndef OPAL_H323_H323CON_H
@@ -229,6 +229,36 @@ class H323Connection : public OpalRTPConnection
       */
     virtual PString GetDestinationAddress();
 
+    /**Get alerting type information of an incoming call.
+       The type of "distinctive ringing" for the call. The string is protocol
+       dependent, so the caller would need to be aware of the type of call
+       being made. Some protocols may ignore the field completely.
+
+       For SIP this corresponds to the string contained in the "Alert-Info"
+       header field of the INVITE. This is typically a URI for the ring file.
+
+       For H.323 this must be a string representation of an integer from 0 to 7
+       which will be contained in the Q.931 SIGNAL (0x34) Information Element.
+
+       Default behaviour returns an empty string.
+      */
+    virtual PString GetAlertingType() const;
+
+    /**Set alerting type information for outgoing call.
+       The type of "distinctive ringing" for the call. The string is protocol
+       dependent, so the caller would need to be aware of the type of call
+       being made. Some protocols may ignore the field completely.
+
+       For SIP this corresponds to the string contained in the "Alert-Info"
+       header field of the INVITE. This is typically a URI for the ring file.
+
+       For H.323 this must be a string representation of an integer from 0 to 7
+       which will be contained in the Q.931 SIGNAL (0x34) Information Element.
+
+       Default behaviour returns false.
+      */
+    virtual bool SetAlertingType(const PString & info);
+
     /**Get the data formats this connection is capable of operating.
        This provides a list of media data format names that an
        OpalMediaStream may be created in within this connection.
@@ -287,6 +317,8 @@ class H323Connection : public OpalRTPConnection
        H323EndPoint during a clear call.
       */
     virtual void CleanUpOnCallEnd();
+
+    virtual void ApplyStringOptions(OpalConnection::StringOptions & stringOptions);
   //@}
 
 
@@ -1773,10 +1805,13 @@ class H323Connection : public OpalRTPConnection
     unsigned GetCallReference() const { return callReference; }
 
     /**Get the call identifier for this connection.
-      * preserved for backwards compatibility with OpenH323 only
      */
     inline const OpalGloballyUniqueID & GetCallIdentifier() const 
-    { return GetIdentifier(); }
+    { return callIdentifier; }
+
+    /**Get the protocol-specific unique identifier for this connection.
+     */
+    virtual PString GetIdentifier() const;
 
     /**Get the conference identifier for this connection.
      */
@@ -1933,6 +1968,7 @@ class H323Connection : public OpalRTPConnection
     PBoolean                 gatekeeperRouted;
     unsigned             distinctiveRing;
     unsigned             callReference;
+    OpalGloballyUniqueID callIdentifier;
     OpalGloballyUniqueID conferenceIdentifier;
 
     PString            localDestinationAddress;
