@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21283 $
+ * $Revision: 21720 $
  * $Author: rjongbloed $
- * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
+ * $Date: 2008-12-09 01:40:33 +0000 (Tue, 09 Dec 2008) $
  */
 
 #ifndef OPAL_H323_H323EP_H
@@ -846,6 +846,12 @@ class H323EndPoint : public OpalRTPEndPoint
 
   /**@name Member variable access */
   //@{
+    /**Set the default local party name for all connections on this endpoint.
+      */
+    virtual void SetDefaultLocalPartyName(
+      const PString & name  /// Name for local party
+    );
+
     /**Set the user name to be used for the local end of any connections. This
        defaults to the logged in user as obtained from the
        PProcess::GetUserName() function.
@@ -990,26 +996,6 @@ class H323EndPoint : public OpalRTPEndPoint
       */
     virtual void OnReceivedInitiateReturnError();
 
-#if OPAL_VIDEO
-    /**See if should auto-start receive video channels on connection.
-     */
-    PBoolean CanAutoStartReceiveVideo() const { return manager.CanAutoStartReceiveVideo(); }
-
-    /**See if should auto-start transmit video channels on connection.
-     */
-    PBoolean CanAutoStartTransmitVideo() const { return manager.CanAutoStartTransmitVideo(); }
-#endif
-
-#if OPAL_T38_CAPABILITY
-    /**See if should auto-start receive fax channels on connection.
-     */
-    PBoolean CanAutoStartReceiveFax() const { return autoStartReceiveFax; }
-
-    /**See if should auto-start transmit fax channels on connection.
-     */
-    PBoolean CanAutoStartTransmitFax() const { return autoStartTransmitFax; }
-#endif
-
     /**See if should automatically do call forward of connection.
      */
     PBoolean CanAutoCallForward() const { return autoCallForward; }
@@ -1340,9 +1326,6 @@ class H323EndPoint : public OpalRTPEndPoint
     // Configuration variables, commonly changed
     PStringList     localAliasNames;
     PStringList     localAliasPatterns;
-    PBoolean        autoStartReceiveFax;
-    PBoolean        autoStartTransmitFax;
-    PBoolean        isH224Enabled;
     PBoolean        autoCallForward;
     PBoolean        disableFastStart;
     PBoolean        disableH245Tunneling;
