@@ -24,9 +24,9 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 21317 $
+ * $Revision: 21909 $
  * $Author: dereksmithies $
- * $Date: 2008-10-13 21:35:37 +0000 (Mon, 13 Oct 2008) $
+ * $Date: 2009-01-09 03:48:34 +0000 (Fri, 09 Jan 2009) $
  */
 
 #ifndef OPAL_IAX2_IAX2EP_H
@@ -394,6 +394,8 @@ class IAX2EndPoint : public OpalEndPoint
     const PString & transport = PString::Empty()
   );
   
+  /**Report if this iax2 endpoint class is correctly initialised */
+  PBoolean InitialisedOK() { return (transmitter != NULL) && (receiver != NULL); }
   //@}
   
  protected:
