@@ -28,9 +28,9 @@
  *     US Joint Forces Command Joint Concept Development & Experimentation (J9)
  *     http://www.jfcom.mil/about/abt_j9.htm
  *
- * $Revision: 21492 $
- * $Author: rjongbloed $
- * $Date: 2008-11-10 05:41:46 +0000 (Mon, 10 Nov 2008) $
+ * $Revision: 22091 $
+ * $Author: csoutheren $
+ * $Date: 2009-02-20 05:22:44 +0000 (Fri, 20 Feb 2009) $
  */
 
 #ifndef OPAL_OPAL_TRANSPORT_H
@@ -55,9 +55,9 @@ class OpalInternalTransport;
 
 ////////////////////////////////////////////////////////////////
 
-class OpalTransportAddress : public PString
+class OpalTransportAddress : public PCaselessString
 {
-  PCLASSINFO(OpalTransportAddress, PString);
+  PCLASSINFO(OpalTransportAddress, PCaselessString);
   public:
   /**@name Construction */
   //@{
@@ -106,6 +106,7 @@ class OpalTransportAddress : public PString
        Returns PFalse, if the address is not an IP transport address.
       */
     PBoolean GetIpAndPort(PIPSocket::Address & ip, WORD & port) const;
+    PBoolean GetIpAndPort(PIPSocketAddressAndPort & ipPort) const;
 
     /**Translate the transport address to a more human readable form.
        Returns the hostname if using IP.
