@@ -24,13 +24,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21004 $
+ * $Revision: 22097 $
  * $Author: rjongbloed $
- * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
+ * $Date: 2009-02-21 04:55:10 +0000 (Sat, 21 Feb 2009) $
  */
 
-#ifndef __OPAL_SIPPDU_H
-#define __OPAL_SIPPDU_H
+#ifndef OPAL_SIP_SIPPDU_H
+#define OPAL_SIP_SIPPDU_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -38,6 +38,8 @@
 
 #include <opal/buildopts.h>
 
+#if OPAL_SIP
+
 #include <ptclib/mime.h>
 #include <ptclib/url.h>
 #include <sip/sdp.h>
@@ -94,6 +96,11 @@ class SIPURL : public PURL
       WORD listenerPort = 0
     );
 
+    SIPURL(
+      const OpalTransportAddress & _address, 
+      WORD listenerPort = 0
+    );
+
     /**Compare the two SIPURLs and return their relative rank.
        Note that does an intelligent comparison according to the rules
        in RFC3261 Section 19.1.4.
@@ -122,7 +129,11 @@ class SIPURL : public PURL
       */
     PString GetFieldParameters() const { return fieldParameters; }
 
-    /**Get the host and port as a transpoprt address.
+    /**Returns the field parameter (outside of <>)
+      */
+    void SetFieldParameters(const PString & str ) { fieldParameters = str; }
+
+    /**Get the host and port as a transport address.
       */
     OpalTransportAddress GetHostAddress() const;
 
@@ -132,7 +143,8 @@ class SIPURL : public PURL
       ToURI,        ///< To header field
       FromURI,      ///< From header field
       ContactURI,   ///< Registration or Redirection Contact header field
-      RouteURI      ///< Dialog Contact header field, or Record-Route header field
+      RouteURI,     ///< Dialog Contact header field, or Record-Route header field
+      RegisterURI   ///< URI on REGISTER request line.
     };
 
     /** Removes tag parm & query vars and recalculates urlString
@@ -153,7 +165,17 @@ class SIPURL : public PURL
       PINDEX entry = 0  /// Entry in the SRV record to adjust to
     );
 
+    /// Generate a unique string suitable as a dialog tag
+    static PString GenerateTag();
+
+    /// Set a tag with a new unique ID.
+    void SetTag(
+      const PString & tag = GenerateTag()
+    );
+
   protected:
+    void ParseAsAddress(const PString & name, const OpalTransportAddress & _address, WORD listenerPort = 0);
+
     /** Parses name-addr, like:
         "displayname"<scheme:user:password@host:port;transport=type>;tag=value
         into:
@@ -257,7 +279,7 @@ class SIPMIMEInfo : public PMIMEInfo
     void SetCallID(const PString & v);
 
     PString GetContact() const;
-    bool GetContacts(std::vector<SIPURL> & contacts) const;
+    bool GetContacts(std::list<SIPURL> & contacts) const;
     void SetContact(const PString & v);
     void SetContact(const SIPURL & url);
 
@@ -304,9 +326,10 @@ class SIPMIMEInfo : public PMIMEInfo
     void SetProxyAuthenticate(const PString & v);
 
     PStringList GetRoute() const;
+    void SetRoute(const PString & v);
     void SetRoute(const PStringList & v);
 
-    PStringList GetRecordRoute() const;
+    PStringList GetRecordRoute(bool reversed) const;
     void SetRecordRoute(const PStringList & v);
 
     unsigned GetCSeqIndex() const { return GetCSeq().AsUnsigned(); }
@@ -341,6 +364,12 @@ class SIPMIMEInfo : public PMIMEInfo
     PString GetSIPETag() const;
     void SetSIPETag(const PString & v);
 
+    PString GetRequire() const;
+    void SetRequire(const PString & v, bool overwrite);
+
+    void GetAlertInfo(PString & info, int & appearance);
+    void SetAlertInfo(const PString & info, int appearance);
+
     /** return the value of a header field parameter, empty if none
      */
     PString GetFieldParameter(
@@ -380,7 +409,7 @@ class SIPMIMEInfo : public PMIMEInfo
   protected:
     	/** return list of route values from internal comma-delimited list
 	 */
-    PStringList GetRouteList(const char * name) const;
+    PStringList GetRouteList(const char * name, bool reversed) const;
 
 	/** store string list as one comma-delimited string of route values
 	    value formed as "<v[0]>,<v[1]>,<v[2]>" etc
@@ -401,9 +430,9 @@ class SIPAuthentication : public PObject
   public:
     SIPAuthentication();
 
-    virtual bool EquivalentTo(
-      const SIPAuthentication & _oldAuth
-    ) = 0;
+    virtual Comparison Compare(
+      const PObject & other
+    ) const;
 
     virtual PBoolean Parse(
       const PString & auth,
@@ -418,11 +447,11 @@ class SIPAuthentication : public PObject
 
     virtual PString GetUsername() const   { return username; }
     virtual PString GetPassword() const   { return password; }
-    virtual PString GetAuthRealm() const  { return authRealm; }
+    virtual PString GetAuthRealm() const  { return PString::Empty(); }
 
     virtual void SetUsername(const PString & user) { username = user; }
     virtual void SetPassword(const PString & pass) { password = pass; }
-    virtual void SetAuthRealm(const PString & r)   { authRealm = r; }
+    virtual void SetAuthRealm(const PString &)     { }
 
     PString GetAuthParam(const PString & auth, const char * name) const;
     PString AsHex(PMessageDigest5::Code & digest) const;
@@ -437,7 +466,6 @@ class SIPAuthentication : public PObject
 
     PString   username;
     PString   password;
-    PString   authRealm;
 };
 
 typedef PFactory<SIPAuthentication> SIPAuthenticationFactory;
@@ -454,19 +482,22 @@ class SIPDigestAuthentication : public SIPAuthentication
       const SIPDigestAuthentication & auth
     );
 
-    bool EquivalentTo(
-      const SIPAuthentication & _oldAuth
-    );
+    virtual Comparison Compare(
+      const PObject & other
+    ) const;
 
-    PBoolean Parse(
+    virtual PBoolean Parse(
       const PString & auth,
       PBoolean proxy
     );
 
-    PBoolean Authorise(
+    virtual PBoolean Authorise(
       SIP_PDU & pdu
     ) const;
 
+    virtual PString GetAuthRealm() const         { return authRealm; }
+    virtual void SetAuthRealm(const PString & r) { authRealm = r; }
+
     enum Algorithm {
       Algorithm_MD5,
       NumAlgorithms
@@ -476,6 +507,7 @@ class SIPDigestAuthentication : public SIPAuthentication
     const PString & GetOpaque() const      { return opaque; }
 
   protected:
+    PString   authRealm;
     PString   nonce;
     Algorithm algorithm;
     PString   opaque;
@@ -651,7 +683,8 @@ class SIP_PDU : public PSafeObject
       If first route is strict, exchange with URI.
       Returns PTrue if routeSet.
       */
-    PBoolean SetRoute(const PStringList & routeSet);
+    bool SetRoute(const PStringList & routeSet);
+    bool SetRoute(const SIPURL & proxy);
 
     /**Set mime allow field to all supported methods.
       */
@@ -671,7 +704,8 @@ class SIP_PDU : public PSafeObject
       */
     PBoolean Write(
       OpalTransport & transport,
-      const OpalTransportAddress & remoteAddress = OpalTransportAddress()
+      const OpalTransportAddress & remoteAddress = OpalTransportAddress(),
+      const PString & localInterface = PString::Empty()
     );
 
     /**Write PDU as a response to a request.
@@ -679,12 +713,14 @@ class SIP_PDU : public PSafeObject
     bool SendResponse(
       OpalTransport & transport,
       StatusCodes code,
+      SIPEndPoint * endpoint = NULL,
       const char * contact = NULL,
       const char * extra = NULL
     );
     bool SendResponse(
       OpalTransport & transport,
-      SIP_PDU & response
+      SIP_PDU & response,
+      SIPEndPoint * endpoint = NULL
     );
 
     /** Construct the PDU string to output.
@@ -731,6 +767,61 @@ ostream & operator<<(ostream & strm, SIP_PDU::Methods method);
 #endif
 
 
+/////////////////////////////////////////////////////////////////////////
+// SIPDialogContext
+
+/** Session Initiation Protocol dialog context information.
+  */
+class SIPDialogContext
+{
+  public:
+    SIPDialogContext();
+
+    const PString & GetCallID() const { return m_callId; }
+    void SetCallID(const PString & id) { m_callId = id; }
+
+    const SIPURL & GetRequestURI() const { return m_requestURI; }
+    void SetRequestURI(const SIPURL & url) { m_requestURI = url; }
+    bool SetRequestURI(const PString & uri) { return m_requestURI.Parse(uri); }
+
+    const PString & GetLocalTag() const { return m_localTag; }
+    void SetLocalTag(const PString & tag) { m_localTag = tag; }
+
+    const SIPURL & GetLocalURI() const { return m_localURI; }
+    void SetLocalURI(const SIPURL & url);
+    bool SetLocalURI(const PString & uri);
+
+    const PString & GetRemoteTag() const { return m_remoteTag; }
+    void SetRemoteTag(const PString & tag) { m_remoteTag = tag; }
+
+    const SIPURL & GetRemoteURI() const { return m_remoteURI; }
+    void SetRemoteURI(const SIPURL & url);
+    bool SetRemoteURI(const PString & uri);
+
+    const PStringList & GetRouteSet() const { return m_routeSet; }
+    void SetRouteSet(const PStringList & routes) { m_routeSet = routes; }
+    void UpdateRouteSet(const SIPURL & proxy);
+
+    void Update(const SIP_PDU & response);
+
+    unsigned GetNextCSeq() { return ++m_lastSentCSeq; }
+    bool IsDuplicateCSeq(unsigned sequenceNumber);
+
+    bool IsEstablished() const { return !m_remoteTag.IsEmpty(); }
+
+  protected:
+    PString     m_callId;
+    SIPURL      m_requestURI;
+    SIPURL      m_localURI;
+    PString     m_localTag;
+    SIPURL      m_remoteURI;
+    PString     m_remoteTag;
+    PStringList m_routeSet;
+    unsigned    m_lastSentCSeq;
+    unsigned    m_lastReceivedCSeq;
+};
+
+
 /////////////////////////////////////////////////////////////////////////
 // SIPTransaction
 
@@ -765,11 +856,13 @@ class SIPTransaction : public SIP_PDU
     ~SIPTransaction();
 
     PBoolean Start();
-    PBoolean IsInProgress() const { return state == Trying || state == Proceeding; }
-    PBoolean IsFailed() const { return state > Terminated_Success; }
-    PBoolean IsCompleted() const { return state >= Completed; }
-    PBoolean IsCanceled() const { return state == Cancelling || state == Terminated_Cancelled || state == Terminated_Aborted; }
-    PBoolean IsTerminated() const { return state >= Terminated_Success; }
+    bool IsTrying()     const { return state == Trying; }
+    bool IsProceeding() const { return state == Proceeding; }
+    bool IsInProgress() const { return state == Trying || state == Proceeding; }
+    bool IsFailed()     const { return state > Terminated_Success; }
+    bool IsCompleted()  const { return state >= Completed; }
+    bool IsCanceled()   const { return state == Cancelling || state == Terminated_Cancelled || state == Terminated_Aborted; }
+    bool IsTerminated() const { return state >= Terminated_Success; }
 
     void WaitForCompletion();
     PBoolean Cancel();
@@ -780,13 +873,19 @@ class SIPTransaction : public SIP_PDU
 
     OpalTransport & GetTransport() const  { return transport; }
     SIPConnection * GetConnection() const { return connection; }
-    PString         GetInterface() const { return localInterface; }
+    PString         GetInterface() const { return m_localInterface; }
 
+    static PString GenerateCallID();
+    
   protected:
     void Construct(
       const PTimeInterval & minRetryTime = PMaxTimeInterval,
       const PTimeInterval & maxRetryTime = PMaxTimeInterval
     );
+    void Construct(
+      Methods method, 
+      SIPDialogContext & dialog
+    );
     bool SendPDU(SIP_PDU & pdu);
     bool ResendCANCEL();
 
@@ -796,7 +895,6 @@ class SIPTransaction : public SIP_PDU
     enum States {
       NotStarted,
       Trying,
-      Aborting,
       Proceeding,
       Cancelling,
       Completed,
@@ -821,7 +919,7 @@ class SIPTransaction : public SIP_PDU
     PTimer     retryTimer;
     PTimer     completionTimer;
     PSyncPoint completed;
-    PString    localInterface;
+    PString              m_localInterface;
     OpalTransportAddress m_remoteAddress;
 };
 
@@ -841,12 +939,12 @@ class SIPInvite : public SIPTransaction
     SIPInvite(
       SIPConnection & connection,
       OpalTransport & transport,
-      OpalRTPSessionManager * sm
+      const OpalRTPSessionManager & sm
     );
 
     virtual PBoolean OnReceivedResponse(SIP_PDU & response);
 
-    OpalRTPSessionManager & GetSessionManager() { return rtpSessions; }
+    const OpalRTPSessionManager & GetSessionManager() const { return rtpSessions; }
 
   protected:
     OpalRTPSessionManager rtpSessions;
@@ -877,8 +975,9 @@ class SIPRegister : public SIPTransaction
     SIPRegister(
       SIPEndPoint   & endpoint,
       OpalTransport & transport,
-      const PStringList & routeSet,
-      const PString & id,
+      const SIPURL & proxy,
+      const PString & callId,
+      unsigned cseq,
       const Params & params
     );
 };
@@ -895,17 +994,29 @@ class SIPSubscribe : public SIPTransaction
     enum PredefinedPackages {
       MessageSummary,
       Presence,
+      Dialog,
       NumPredefinedPackages
     };
-    static PredefinedPackages GetEventPackage(const PString & name);
-    static PString GetEventPackageName(PredefinedPackages pkg);
+
+    class EventPackage : public PCaselessString
+    {
+        PCLASSINFO(EventPackage, PCaselessString);
+      public:
+        EventPackage(PredefinedPackages);
+        EventPackage(const PString & str) : PCaselessString(str) { }
+        EventPackage(const char * cstr) : PCaselessString(cstr) { }
+        bool operator==(PredefinedPackages) const;
+        bool operator==(const PString & str) const { return Compare(str) == EqualTo; }
+        bool operator==(const char * cstr) const { return InternalCompare(0, P_MAX_INDEX, cstr) == EqualTo; }
+        virtual Comparison InternalCompare(PINDEX offset, PINDEX length, const char * cstr) const;
+    };
 
     struct Params {
       Params(PredefinedPackages pkg = NumPredefinedPackages);
 
-      PString       m_eventPackage;
-      PString       m_mimeType;
-      PString       m_targetAddress;
+      EventPackage  m_eventPackage;
+      PString       m_agentAddress;
+      PString       m_addressOfRecord;
       PString       m_contactAddress;
       PString       m_authID;
       PString       m_password;
@@ -916,21 +1027,52 @@ class SIPSubscribe : public SIPTransaction
       PTimeInterval m_maxRetryTime;
     };
 
-    /** Valid types for a MWI
-    */
     SIPSubscribe(
         SIPEndPoint & ep,
         OpalTransport & trans,
-        const PStringList & routeSet,
-        const PString & to,
-        const PString & from,
-        const PString & id,
-        unsigned cseq,
+        SIPDialogContext & dialog,
         const Params & params
     );
 };
 
 
+typedef SIPSubscribe::EventPackage SIPEventPackage;
+
+
+/////////////////////////////////////////////////////////////////////////
+
+class SIPHandler;
+
+class SIPEventPackageHandler
+{
+public:
+  virtual ~SIPEventPackageHandler() { }
+  virtual PString GetContentType() const = 0;
+  virtual bool OnReceivedNOTIFY(SIPHandler & handler, SIP_PDU & request) = 0;
+  virtual PString OnSendNOTIFY(SIPHandler & /*handler*/, const PObject * /*body*/) { return PString::Empty(); }
+};
+
+
+typedef PFactory<SIPEventPackageHandler, SIPEventPackage> SIPEventPackageFactory;
+
+
+/////////////////////////////////////////////////////////////////////////
+
+class SIPNotify : public SIPTransaction
+{
+    PCLASSINFO(SIPNotify, SIPTransaction);
+  public:
+    SIPNotify(
+        SIPEndPoint & ep,
+        OpalTransport & trans,
+        SIPDialogContext & dialog,
+        const SIPEventPackage & eventPackage,
+        const PString & state,
+        const PString & body
+    );
+};
+
+
 /////////////////////////////////////////////////////////////////////////
 
 class SIPPublish : public SIPTransaction
@@ -940,12 +1082,10 @@ class SIPPublish : public SIPTransaction
     SIPPublish(
       SIPEndPoint & ep,
       OpalTransport & trans,
-      const PStringList & routeSet,
-      const SIPURL & targetAddress,
       const PString & id,
       const PString & sipIfMatch,
-      const PString & body,
-      unsigned expires
+      SIPSubscribe::Params & params,
+      const PString & body
     );
 };
 
@@ -1004,12 +1144,12 @@ class SIPMessage : public SIPTransaction
     
   public:
     SIPMessage(
-	       SIPEndPoint & ep,
-	       OpalTransport & trans,
-	       const SIPURL & to,
-               const PStringList & routeSet,
-               const PString & id,
-	       const PString & body
+      SIPEndPoint & ep,
+      OpalTransport & trans,
+      const SIPURL & proxy,
+      const SIPURL & to,
+      const PString & id,
+      const PString & body
     );
 };
 
@@ -1065,7 +1205,9 @@ class SIPPing : public SIPTransaction
 };
 
 
-#endif // __OPAL_SIPPDU_H
+#endif // OPAL_SIP
+
+#endif // OPAL_SIP_SIPPDU_H
 
 
 // End of File ///////////////////////////////////////////////////////////////
