@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 21795 $
+ * $Author: rjongbloed $
+ * $Date: 2008-12-14 23:15:22 +0000 (Sun, 14 Dec 2008) $
  */
 
 #include "precompile.h"
@@ -40,6 +40,7 @@ const WORD DefaultHTTPPort = 1719;
 static const char UsernameKey[] = "Username";
 static const char PasswordKey[] = "Password";
 static const char LogLevelKey[] = "Log Level";
+static const char DefaultAddressFamilyKey[] = "AddressFamily";
 #if OPAL_PTLIB_SSL
 static const char HTTPCertificateFileKey[]  = "HTTP Certificate";
 #endif
@@ -55,6 +56,7 @@ static const char UDPPortMaxKey[] = "UDP Port Max";
 static const char RTPPortBaseKey[] = "RTP Port Base";
 static const char RTPPortMaxKey[] = "RTP Port Max";
 static const char RTPTOSKey[] = "RTP Type of Service";
+static const char STUNServerKey[] = "STUN Server";
 
 static const char H323AliasesKey[] = "H.323 Aliases";
 static const char DisableFastStartKey[] = "Disable Fast Start";
@@ -167,6 +169,10 @@ PBoolean OpalGw::Initialise(const char * initMsg)
   PString username = cfg.GetString(UsernameKey);
   PString password = PHTTPPasswordField::Decrypt(cfg.GetString(PasswordKey));
 
+  PString addressFamily = cfg.GetString(DefaultAddressFamilyKey, "IPV4");
+  if(addressFamily *= "IPV6")
+	 PIPSocket::SetDefaultIpAddressFamilyV6();
+
   PHTTPSimpleAuth authority(GetName(), username, password);
 
   // Create the parameters URL page, and start adding fields to it
@@ -184,7 +190,7 @@ PBoolean OpalGw::Initialise(const char * initMsg)
 
 #if OPAL_PTLIB_SSL
   // SSL certificate file.
-  PString certificateFile = cfg.GetString(HTTPCertificateFileKey, "server.pem");
+  PString certificateFile = cfg.GetString(HTTPCertificateFileKey);
   rsrc->Add(new PHTTPStringField(HTTPCertificateFileKey, 25, certificateFile));
   if (certificateFile.IsEmpty())
     disableSSL = true;
@@ -278,7 +284,8 @@ MyManager::MyManager()
 #endif
 
 #if OPAL_VIDEO
-  autoStartReceiveVideo = autoStartTransmitVideo = PFalse;
+  SetAutoStartReceiveVideo(false);
+  SetAutoStartTransmitVideo(false);
 #endif
 }
 
@@ -360,6 +367,11 @@ PBoolean MyManager::Initialise(PConfig & cfg, PConfigPage * rsrc)
   SetRtpIpTypeofService(cfg.GetInteger(RTPTOSKey, GetRtpIpTypeofService()));
   rsrc->Add(new PHTTPIntegerField(RTPTOSKey,  0, 255, GetRtpIpTypeofService()));
 
+  PString STUNServer = cfg.GetString(STUNServerKey, "stun.voxgratia.org");
+  rsrc->Add(new PHTTPStringField(STUNServerKey, 25, STUNServer));
+  if(PIPSocket::GetDefaultIpAddressFamily() != AF_INET6)
+	SetSTUNServer(STUNServer);
+
 #if OPAL_H323
 
   // Add H.323 parameters
@@ -584,5 +596,4 @@ PBoolean MyManager::Initialise(PConfig & cfg, PConfigPage * rsrc)
   return PTrue;
 }
 
-
 // End of File ///////////////////////////////////////////////////////////////
