@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21857 $
- * $Author: rjongbloed $
- * $Date: 2008-12-22 04:03:10 +0000 (Mon, 22 Dec 2008) $
+ * $Revision: 21910 $
+ * $Author: dereksmithies $
+ * $Date: 2009-01-09 03:50:24 +0000 (Fri, 09 Jan 2009) $
  */
 
 #include <ptlib.h>
@@ -341,7 +341,8 @@ void SimpleOpalProcess::Main()
 #endif
 #if OPAL_IAX2
 "    If IAX2 is enabled then you can make a iax2 call with a command like:\n"
-"       simpleopal -IHn  iax2:guest@misery.digium.com/s\n"
+"       simpleopal -I -H  iax2:guest@misery.digium.com/s\n"
+"           ((Please ensure simplopal is the only iax2 app running on your box))\n"
 #endif
             << endl;
     return;
@@ -605,6 +606,12 @@ PBoolean MyManager::Initialise(PArgList & args)
   if (!args.HasOption("no-iax2")) {
     iax2EP = new IAX2EndPoint(*this);
     
+    if (!iax2EP->InitialisedOK()) {
+      cerr << "IAX2 Endpoint is not initialised correctly" << endl;
+      cerr << "Is there another application using port 4569? " << endl;
+      return PFalse;
+    }
+
     if (args.HasOption('p'))
       iax2EP->SetPassword(args.GetOptionString('p'));
     
@@ -842,6 +849,7 @@ PBoolean MyManager::Initialise(PArgList & args)
         unsigned rate = args.GetOptionString("video-rate").AsUnsigned();
         unsigned frameTime = 90000 / rate;
         mediaFormat.SetOptionInteger(OpalMediaFormat::FrameTimeOption(), frameTime);
+        mediaFormat.SetOptionBoolean("Rate Control Enable", true);
       }
       OpalMediaFormat::SetRegisteredMediaFormat(mediaFormat);
     }
