@@ -24,9 +24,9 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 21283 $
- * $Author: rjongbloed $
- * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
+ * $Revision: 21421 $
+ * $Author: dereksmithies $
+ * $Date: 2008-10-30 00:36:33 +0000 (Thu, 30 Oct 2008) $
  */
 
 #include <ptlib.h>
@@ -1562,14 +1562,12 @@ void IAX2FrameList::DeleteMatchingSendFrame(IAX2FullFrame *reply)
       continue;
     } 
 
-    if (sent->GetSequenceInfo().IsSequenceNosZero() && sent->IsNewFrame()) {
-      //Outgoing frame has a zero sequence nos, - delete this one
-      PTRACE(5, "MarkDeleteNow on the New frame " << sent->IdString());
-      PTRACE(5, "Have a reply of " << reply->IdString());
+    if (sent->IsNewFrame() &&
+	reply->GetSequenceInfo().IsFirstReplyFrame()) {
+      PTRACE(5, "Have a match on a new frame we sent out");
       goto foundMatch;
-    } else {
-      PTRACE(5, "Non zero sequence nos in the sequence number - look for exact match");
     }
+
         
     if (sent->IsRegReqFrame() && 
         (reply->IsRegAckFrame() || reply->IsRegAuthFrame() || reply->IsRegRejFrame())) {
