@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21655 $
+ * $Revision: 22096 $
  * $Author: rjongbloed $
- * $Date: 2008-11-30 05:16:47 +0000 (Sun, 30 Nov 2008) $
+ * $Date: 2009-02-21 03:25:23 +0000 (Sat, 21 Feb 2009) $
  */
 
 #include <ptlib.h>
@@ -77,8 +77,6 @@ OpalCall::OpalCall(OpalManager & mgr)
 OpalCall::~OpalCall()
 {
   PTRACE(3, "Call\t" << *this << " destroyed.");
-
-  manager.GetRecordManager().Close(myToken);
 }
 
 
@@ -133,6 +131,7 @@ void OpalCall::Clear(OpalConnection::CallEndReason reason, PSyncPoint * sync)
 void OpalCall::OnCleared()
 {
   manager.OnClearedCall(*this);
+  manager.GetRecordManager().Close(myToken);
 
   if (!LockReadWrite())
     return;
@@ -174,6 +173,12 @@ PBoolean OpalCall::OnSetUp(OpalConnection & connection)
 }
 
 
+void OpalCall::OnProceeding(OpalConnection & PTRACE_PARAM(connection))
+{
+  PTRACE(3, "Call\tOnProceeding " << connection);
+}
+
+
 PBoolean OpalCall::OnAlerting(OpalConnection & connection)
 {
   PTRACE(3, "Call\tOnAlerting " << connection);
@@ -388,12 +393,15 @@ PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection,
     OpalMediaPatch * patch = sourceStream->GetPatch();
     if (patch != NULL)
       sinkStream = patch->GetSink();
-    if (!preselectedFormat.IsValid() ||
-         preselectedFormat == sourceStream->GetMediaFormat() ||
-         (sinkStream != NULL && sinkStream->GetMediaFormat() == preselectedFormat)) {
+    if (sourceStream->GetMediaFormat() == preselectedFormat ||
+        (sinkStream != NULL && sinkStream->GetMediaFormat() == preselectedFormat)) {
       PTRACE(3, "Call\tOpenSourceMediaStreams (already opened) for session " << sessionID << " on " << connection);
       return true;
     }
+    if (sourceStream->IsOpen()) {
+      PTRACE(3, "Call\tOpenSourceMediaStreams (is opening) for session " << sessionID << " on " << connection);
+      return true;
+    }
   }
 
   PTRACE(3, "Call\tOpenSourceMediaStreams " << (sourceStream != NULL ? "replacing" : "opening")
@@ -588,10 +596,10 @@ void OpalCall::OnHold(OpalConnection & /*connection*/,
 }
 
 
-PBoolean OpalCall::StartRecording(const PFilePath & fn)
+PBoolean OpalCall::StartRecording(const PFilePath & fn, bool mono)
 {
   // create the mixer entry
-  if (!manager.GetRecordManager().Open(myToken, fn))
+  if (!manager.GetRecordManager().Open(myToken, fn, mono))
     return PFalse;
 
   // tell each connection to start sending data
@@ -605,7 +613,7 @@ PBoolean OpalCall::StartRecording(const PFilePath & fn)
 
 bool OpalCall::IsRecording() const
 {
-  return manager.GetRecordManager().IsOpen();
+  return manager.GetRecordManager().IsOpen(myToken);
 }
 
 
@@ -620,9 +628,16 @@ void OpalCall::StopRecording()
 }
 
 
-void OpalCall::OnStopRecordAudio(const PString & callToken)
+void OpalCall::OnRecordAudio(const PString & streamId, const RTP_DataFrame & frame)
 {
-  manager.GetRecordManager().CloseStream(myToken, callToken);
+  if (!manager.GetRecordManager().WriteAudio(myToken, streamId, frame))
+    StopRecording();
+}
+
+
+void OpalCall::OnStopRecordAudio(const PString & streamId)
+{
+  manager.GetRecordManager().CloseStream(myToken, streamId);
 }
 
 
@@ -654,14 +669,17 @@ void OpalCall::SetPartyNames()
     return;
 
   if (connectionB->IsNetworkConnection()) {
-    m_partyB = connectionB->GetRemotePartyURL();
-    if (!connectionA->IsNetworkConnection())
+    if (!networkA)
       connectionA->CopyPartyNames(*connectionB);
+    m_partyB = connectionB->GetRemotePartyURL();
   }
   else {
-    m_partyB = connectionB->GetLocalPartyURL();
-    if (connectionA->IsNetworkConnection())
+    if (networkA) {
       connectionB->CopyPartyNames(*connectionA);
+      m_partyB = connectionA->GetCalledPartyURL();
+    }
+    if (m_partyB.IsEmpty())
+      m_partyB = connectionB->GetLocalPartyURL();
   }
 }
 
