@@ -23,9 +23,9 @@
  *
  * Contributor(s): ________________________________________.
  *
- * $Revision: 21866 $
- * $Author: csoutheren $
- * $Date: 2008-12-24 00:15:43 +0000 (Wed, 24 Dec 2008) $
+ * $Revision: 22089 $
+ * $Author: rjongbloed $
+ * $Date: 2009-02-19 04:19:55 +0000 (Thu, 19 Feb 2009) $
  */
 
 #include <ptlib.h>
@@ -397,6 +397,7 @@ PBoolean OpalMediaStream::SetDataSize(PINDEX dataSize)
   if (dataSize <= 0)
     return false;
 
+  PTRACE_IF(4, defaultDataSize != dataSize, "Media\tSet data size from " << defaultDataSize << " to " << dataSize);
   defaultDataSize = dataSize;
   return true;
 }
@@ -706,13 +707,13 @@ OpalRawMediaStream::OpalRawMediaStream(OpalConnection & conn,
                                      const OpalMediaFormat & mediaFormat,
                                        unsigned sessionID,
                                        PBoolean isSource,
-                                       PChannel * chan, PBoolean autoDel)
+                                       PChannel * chan, PBoolean autoDelete)
   : OpalMediaStream(conn, mediaFormat, sessionID, isSource)
+  , m_channel(chan)
+  , m_autoDelete(autoDelete)
+  , m_averageSignalSum(0)
+  , m_averageSignalSamples(0)
 {
-  channel = chan;
-  autoDelete = autoDel;
-  averageSignalSum = 0;
-  averageSignalSamples = 0;
 }
 
 
@@ -720,9 +721,9 @@ OpalRawMediaStream::~OpalRawMediaStream()
 {
   Close();
 
-  if (autoDelete)
-    delete channel;
-  channel = NULL;
+  if (m_autoDelete)
+    delete m_channel;
+  m_channel = NULL;
 }
 
 
@@ -738,13 +739,15 @@ PBoolean OpalRawMediaStream::ReadData(BYTE * buffer, PINDEX size, PINDEX & lengt
     return false;
   }
 
-  if (!IsOpen() || channel == NULL)
+  PWaitAndSignal mutex(m_channelMutex);
+
+  if (!IsOpen() || m_channel == NULL)
     return false;
 
-  if (!channel->Read(buffer, size))
+  if (!m_channel->Read(buffer, size))
     return false;
 
-  length = channel->GetLastReadCount();
+  length = m_channel->GetLastReadCount();
   CollectAverage(buffer, length);
   return true;
 }
@@ -764,25 +767,61 @@ PBoolean OpalRawMediaStream::WriteData(const BYTE * buffer, PINDEX length, PINDE
     return false;
   }
   
-  if (!IsOpen() || channel == NULL) {
+  PWaitAndSignal mutex(m_channelMutex);
+
+  if (!IsOpen() || m_channel == NULL) {
     PTRACE(1, "Media\tTried to write to media stream with no channel");
     return false;
   }
 
-  if (buffer != NULL && length != 0) {
-    if (!channel->Write(buffer, length))
-      return false;
-    written = channel->GetLastWriteCount();
-    CollectAverage(buffer, written);
-  }
+  /* We make the assumption that the remote is sending the same sized packets
+     all the time and does not suddenly switch from 30ms to 20ms, even though
+     this is technically legal. We have never yet seen it, and even if it did
+     it doesn't hurt the silence insertion algorithm very much.
+
+     So, silence buffer is set to be the largest chunk of audio the remote has
+     ever sent to us. Then when they stop sending, (we get length==0) we just
+     keep outputting that number of bytes to the raw channel until the remote
+     starts up again.
+     */
+
+  if (buffer != NULL && length != 0)
+    m_silence.SetMinSize(length);
   else {
-    PBYTEArray silence(defaultDataSize);
-    if (!channel->Write(silence, defaultDataSize))
-      return false;
-    written = channel->GetLastWriteCount();
-    CollectAverage(silence, written);
+    length = m_silence.GetSize();
+    buffer = m_silence;
   }
 
+  if (!m_channel->Write(buffer, length))
+    return false;
+
+  written = m_channel->GetLastWriteCount();
+  CollectAverage(buffer, written);
+  return true;
+}
+
+
+bool OpalRawMediaStream::SetChannel(PChannel * chan, bool autoDelete)
+{
+  if (chan == NULL || !chan->IsOpen()) {
+    if (autoDelete)
+      delete chan;
+    return false;
+  }
+
+  m_channelMutex.Wait();
+
+  PChannel * channelToDelete = m_autoDelete ? m_channel : NULL;
+  m_channel = chan;
+  m_autoDelete = autoDelete;
+
+  SetDataSize(GetDataSize());
+
+  m_channelMutex.Signal();
+
+  delete channelToDelete; // Outside mutex
+
+  PTRACE(4, "Media\tSet raw media channel to \"" << m_channel->GetName() << '"');
   return true;
 }
 
@@ -792,8 +831,8 @@ PBoolean OpalRawMediaStream::Close()
   if (!OpalMediaStream::Close())
     return false;
 
-  if (channel != NULL)
-    channel->Close();
+  if (m_channel != NULL)
+    m_channel->Close();
 
   return true;
 }
@@ -801,27 +840,27 @@ PBoolean OpalRawMediaStream::Close()
 
 unsigned OpalRawMediaStream::GetAverageSignalLevel()
 {
-  PWaitAndSignal m(averagingMutex);
+  PWaitAndSignal mutex(m_averagingMutex);
 
-  if (averageSignalSamples == 0)
+  if (m_averageSignalSamples == 0)
     return UINT_MAX;
 
-  unsigned average = (unsigned)(averageSignalSum/averageSignalSamples);
-  averageSignalSum = average;
-  averageSignalSamples = 1;
+  unsigned average = (unsigned)(m_averageSignalSum/m_averageSignalSamples);
+  m_averageSignalSum = average;
+  m_averageSignalSamples = 1;
   return average;
 }
 
 
 void OpalRawMediaStream::CollectAverage(const BYTE * buffer, PINDEX size)
 {
-  PWaitAndSignal m(averagingMutex);
+  PWaitAndSignal mutex(m_averagingMutex);
 
   size = size/2;
-  averageSignalSamples += size;
+  m_averageSignalSamples += size;
   const short * pcm = (const short *)buffer;
   while (size-- > 0) {
-    averageSignalSum += PABS(*pcm);
+    m_averageSignalSum += PABS(*pcm);
     pcm++;
   }
 }
@@ -923,7 +962,7 @@ PBoolean OpalAudioMediaStream::SetDataSize(PINDEX dataSize)
   /* For efficiency reasons we will not accept a packet size that is too small.
      We move it up to the next even multiple, which has a danger of the remote not
      sending an even number of our multiplier. */
-  const unsigned MinTime = 20;
+  const unsigned MinTime = 10;
   PINDEX minSize = mediaFormat.GetClockRate()/(1000/MinTime)*sizeof(short);
   if (dataSize < minSize) {
     PTRACE(1, "Media\tClamping audio stream data size from " << dataSize << " to minimum " << minSize);
@@ -933,7 +972,7 @@ PBoolean OpalAudioMediaStream::SetDataSize(PINDEX dataSize)
   PTRACE(3, "Media\tAudio " << (IsSource() ? "source" : "sink") << " data size set to "
          << dataSize << " bytes and " << soundChannelBuffers << " buffers.");
   return OpalMediaStream::SetDataSize(dataSize) &&
-         ((PSoundChannel *)channel)->SetBuffers(dataSize, soundChannelBuffers);
+         ((PSoundChannel *)m_channel)->SetBuffers(dataSize, soundChannelBuffers);
 }
 
 
@@ -1085,7 +1124,7 @@ PBoolean OpalVideoMediaStream::ReadData(BYTE * data, PINDEX size, PINDEX & lengt
   marker = true;
   length = bytesReturned + sizeof(PluginCodec_Video_FrameHeader);
 
-  if ((flags & PluginCodec_CoderForceIFrame) != 0) {
+  if ((flags & PluginCodec_ReturnCoderRequestIFrame) != 0) {
     ExecuteCommand(OpalVideoUpdatePicture());
   }
 
