@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21860 $
+ * $Revision: 22081 $
  * $Author: rjongbloed $
- * $Date: 2008-12-23 02:24:22 +0000 (Tue, 23 Dec 2008) $
+ * $Date: 2009-02-17 04:53:53 +0000 (Tue, 17 Feb 2009) $
  */
 
 #include <ptlib.h>
@@ -47,10 +47,12 @@
 #include <codec/vidcodec.h>
 #endif
 
+
+#define new PNEW
+
+
 #ifdef OPAL_ZRTP
-
 extern OpalZRTPConnectionInfo * OpalLibZRTPConnInfo_Create();
-
 #endif
 
 
@@ -83,7 +85,9 @@ OpalRTPConnection::OpalRTPConnection(OpalCall & call,
 OpalRTPConnection::~OpalRTPConnection()
 {
   delete rfc2833Handler;
+#if OPAL_T38_CAPABILITY
   delete ciscoNSEHandler;
+#endif
 }
 
 
@@ -127,39 +131,10 @@ RTP_Session * OpalRTPConnection::CreateSession(const OpalTransport & transport,
   transport.GetRemoteAddress().GetIpAddress(remoteAddress);
   PNatMethod * natMethod = manager.GetNatMethod(remoteAddress);
 
-  OpalMediaType mediaType = OpalMediaTypeDefinition::GetMediaTypeForSessionId(sessionID);
-  OpalMediaTypeDefinition * def = mediaType.GetDefinition();
-  if (def == NULL) {
-    PTRACE(1, "RTPCon\tNo definition for media type " << mediaType);
-    return NULL;
-  }
-
   // create an RTP session
-  RTP_UDP * rtpSession    = NULL;
-
-#ifdef OPAL_ZRTP
-  // create ZRTP channel if enabled
-  {
-    PWaitAndSignal m(zrtpConnInfoMutex);
-    if (zrtpEnabled) {
-      if (zrtpConnInfo == NULL)
-        zrtpConnInfo = OpalLibZRTPConnInfo_Create();
-      if (zrtpConnInfo != NULL) {
-        rtpSession = zrtpConnInfo->CreateRTPSession(*this, sessionID, remoteIsNAT);
-        if (rtpSession == NULL) {
-          delete zrtpConnInfo;
-          zrtpConnInfo = NULL;
-        }
-      }
-    }
-  }
-#endif
-
-  if (rtpSession == NULL) {
-    rtpSession = def->CreateRTPSession(*this, sessionID, remoteIsNAT);  
-    if (rtpSession == NULL) 
-      return NULL;
-  }
+  RTP_UDP * rtpSession = CreateRTPSession(sessionID, remoteIsNAT);
+  if (rtpSession == NULL) 
+    return NULL;
 
   WORD firstPort = manager.GetRtpIpPortPair();
   WORD nextPort = firstPort;
@@ -180,6 +155,38 @@ RTP_Session * OpalRTPConnection::CreateSession(const OpalTransport & transport,
   return rtpSession;
 }
 
+
+RTP_UDP * OpalRTPConnection::CreateRTPSession(unsigned sessionID, bool remoteIsNAT)
+{
+  OpalMediaType mediaType = OpalMediaTypeDefinition::GetMediaTypeForSessionId(sessionID);
+  OpalMediaTypeDefinition * def = mediaType.GetDefinition();
+  if (def == NULL) {
+    PTRACE(1, "RTPCon\tNo definition for media type " << mediaType);
+    return NULL;
+  }
+
+#ifdef OPAL_ZRTP
+  // create ZRTP channel if enabled
+  {
+    PWaitAndSignal m(zrtpConnInfoMutex);
+    if (zrtpEnabled) {
+      if (zrtpConnInfo == NULL)
+        zrtpConnInfo = OpalLibZRTPConnInfo_Create();
+      if (zrtpConnInfo != NULL) {
+        RTP_UDP * rtpSession = zrtpConnInfo->CreateRTPSession(*this, sessionID, remoteIsNAT);
+        if (rtpSession != NULL)
+          return rtpSession;
+        delete zrtpConnInfo;
+        zrtpConnInfo = NULL;
+      }
+    }
+  }
+#endif
+
+  return def->CreateRTPSession(*this, sessionID, remoteIsNAT);
+}
+
+
 void OpalRTPConnection::ReleaseSession(unsigned sessionID, PBoolean clearAll/* = PFalse */)
 {
 #ifdef HAS_LIBZRTP
@@ -234,7 +241,7 @@ void OpalRTPConnection::OnMediaCommand(OpalMediaCommand & /*command*/, INT /*ext
 
 void OpalRTPConnection::AttachRFC2833HandlerToPatch(PBoolean isSource, OpalMediaPatch & patch)
 {
-  if (isSource && (rfc2833Handler != NULL || ciscoNSEHandler != NULL)) {
+  if (isSource) {
     OpalRTPMediaStream * mediaStream = dynamic_cast<OpalRTPMediaStream *>(&patch.GetSource());
     if (mediaStream != NULL) {
       RTP_Session & rtpSession = mediaStream->GetRtpSession();
@@ -242,10 +249,12 @@ void OpalRTPConnection::AttachRFC2833HandlerToPatch(PBoolean isSource, OpalMedia
         PTRACE(3, "RTPCon\tAdding RFC2833 receive handler");
         rtpSession.AddFilter(rfc2833Handler->GetReceiveHandler());
       }
+#if OPAL_T38_CAPABILITY
       if (ciscoNSEHandler != NULL) {
         PTRACE(3, "RTPCon\tAdding Cisco NSE receive handler");
         rtpSession.AddFilter(ciscoNSEHandler->GetReceiveHandler());
       }
+#endif
     }
   }
 }
@@ -253,7 +262,10 @@ void OpalRTPConnection::AttachRFC2833HandlerToPatch(PBoolean isSource, OpalMedia
 
 PBoolean OpalRTPConnection::SendUserInputTone(char tone, unsigned duration)
 {
-  if (ciscoNSEHandler->SendToneAsync(tone, duration) ||
+  if (
+#if OPAL_T38_CAPABILITY
+      ciscoNSEHandler->SendToneAsync(tone, duration) ||
+#endif
        rfc2833Handler->SendToneAsync(tone, duration))
     return true;
 
