@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21004 $
+ * $Revision: 21989 $
  * $Author: rjongbloed $
- * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
+ * $Date: 2009-02-04 06:34:43 +0000 (Wed, 04 Feb 2009) $
  */
 
 #include <ptlib.h>
@@ -76,7 +76,7 @@ OpalTranscoder::OpalTranscoder(const OpalMediaFormat & inputMediaFormat,
                                const OpalMediaFormat & outputMediaFormat)
   : OpalMediaFormatPair(inputMediaFormat, outputMediaFormat)
 {
-  maxOutputSize = P_MAX_INDEX; // Just something, usually changed by OpalMediaPatch
+  maxOutputSize = 32768; // Just something, usually changed by OpalMediaPatch
   outputIsRTP = inputIsRTP = PFalse;
   acceptEmptyPayload = false;
   acceptOtherPayloads = false;
@@ -133,14 +133,14 @@ PBoolean OpalTranscoder::ConvertFrames(const RTP_DataFrame & input, RTP_DataFram
 {
   // make sure there is at least one output frame available
   if (output.IsEmpty())
-    output.Append(new RTP_DataFrame);
+    output.Append(new RTP_DataFrame(0, maxOutputSize));
   else {
     while (output.GetSize() > 1)
       output.RemoveAt(1);
   }
 
   // set the output timestamp and marker bit
-  output.front().SetTimestamp(input.GetTimestamp());
+  output.front().SetTimestamp(input.GetTimestamp()*outputMediaFormat.GetClockRate()/inputMediaFormat.GetClockRate());
   output.front().SetMarker(input.GetMarker());
 
   // set the output payload type directly from the output media format
@@ -573,7 +573,8 @@ OpalStreamedTranscoder::OpalStreamedTranscoder(const OpalMediaFormat & inputMedi
 PINDEX OpalStreamedTranscoder::GetOptimalDataFrameSize(PBoolean input) const
 {
   // For streamed codecs a "frame" is one milliseconds worth of data
-  PINDEX size = outputMediaFormat.GetOptionInteger(OpalAudioFormat::TxFramesPerPacketOption(), 1);
+  PINDEX size = outputMediaFormat.GetOptionInteger(input ? OpalAudioFormat::TxFramesPerPacketOption()
+                                                         : OpalAudioFormat::RxFramesPerPacketOption(), 1);
 
   size *= outputMediaFormat.GetClockRate()/1000;            // Convert to milliseconds
   size *= input ? inputBitsPerSample : outputBitsPerSample; // Total bits
