@@ -28,9 +28,9 @@
  *     US Joint Forces Command Joint Concept Development & Experimentation (J9)
  *     http://www.jfcom.mil/about/abt_j9.htm
  *
- * $Revision: 21599 $
- * $Author: rjongbloed $
- * $Date: 2008-11-23 12:42:25 +0000 (Sun, 23 Nov 2008) $
+ * $Revision: 22091 $
+ * $Author: csoutheren $
+ * $Date: 2009-02-20 05:22:44 +0000 (Fri, 20 Feb 2009) $
  */
 
 #include <ptlib.h>
@@ -79,7 +79,7 @@ OpalTransportAddress::OpalTransportAddress()
 OpalTransportAddress::OpalTransportAddress(const char * cstr,
                                            WORD port,
                                            const char * proto)
-  : PString(cstr)
+  : PCaselessString(cstr)
 {
   SetInternalTransport(port, proto);
 }
@@ -88,14 +88,14 @@ OpalTransportAddress::OpalTransportAddress(const char * cstr,
 OpalTransportAddress::OpalTransportAddress(const PString & str,
                                            WORD port,
                                            const char * proto)
-  : PString(str)
+  : PCaselessString(str)
 {
   SetInternalTransport(port, proto);
 }
 
 
 OpalTransportAddress::OpalTransportAddress(const PIPSocket::Address & addr, WORD port, const char * proto)
-  : PString(addr.AsString(true))
+  : PCaselessString(addr.AsString(true))
 {
   SetInternalTransport(port, proto);
 }
@@ -167,6 +167,21 @@ PBoolean OpalTransportAddress::GetIpAndPort(PIPSocket::Address & ip, WORD & port
 }
 
 
+PBoolean OpalTransportAddress::GetIpAndPort(PIPSocketAddressAndPort & ipPort) const
+{
+  if (transport == NULL)
+    return PFalse;
+
+  PIPSocket::Address ip;
+  WORD port = 0;
+  if (!transport->GetIpAndPort(*this, ip, port))
+    return false;
+
+  ipPort.SetAddress(ip, port);
+  return true;
+}
+
+
 OpalListener * OpalTransportAddress::CreateListener(OpalEndPoint & endpoint,
                                                     BindOptions option) const
 {
@@ -691,8 +706,14 @@ OpalListenerUDP::~OpalListenerUDP()
 
 PBoolean OpalListenerUDP::Open(const PNotifier & theAcceptHandler, ThreadMode /*mode*/)
 {
-  if (listenerBundle->Open(listenerPort))
-    return StartThread(theAcceptHandler, SingleThreadMode);
+  if (listenerBundle->Open(listenerPort) && StartThread(theAcceptHandler, SingleThreadMode)) {
+    /* UDP packets need to be handled. Not so much at high speed, but must not be
+       significantly delayed by media threads which are running at HighPriority.
+       This, for example, helps make sure that a SIP BYE is received and processed
+       to kill a call where codecs etc in the media threads are hogging all the CPU. */
+    thread->SetPriority(PThread::HighestPriority);
+    return true;
+  }
 
   PTRACE(1, "Listen\tCould not start any UDP listeners");
   return PFalse;
