@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21835 $
- * $Author: hfriederich $
- * $Date: 2008-12-18 09:00:44 +0000 (Thu, 18 Dec 2008) $
+ * $Revision: 21986 $
+ * $Author: rjongbloed $
+ * $Date: 2009-02-04 04:31:07 +0000 (Wed, 04 Feb 2009) $
  */
 
 #include <ptlib.h>
@@ -73,10 +73,10 @@ static PFactory<RTP_Encoding>::Worker<RTP_Encoding> rtpAVPHandler("rtp/avp");
 
 /////////////////////////////////////////////////////////////////////////////
 
-RTP_DataFrame::RTP_DataFrame(PINDEX sz)
-  : PBYTEArray(MinHeaderSize+sz)
+RTP_DataFrame::RTP_DataFrame(PINDEX payloadSz, PINDEX bufferSz)
+  : PBYTEArray(std::max(bufferSz, MinHeaderSize+payloadSz))
 {
-  payloadSize = sz;
+  payloadSize = payloadSz;
   theArray[0] = '\x80';
 }
 
