@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21807 $
- * $Author: csoutheren $
- * $Date: 2008-12-16 03:28:12 +0000 (Tue, 16 Dec 2008) $
+ * $Revision: 22087 $
+ * $Author: rjongbloed $
+ * $Date: 2009-02-18 03:41:35 +0000 (Wed, 18 Feb 2009) $
  */
 
 #include <ptlib.h>
@@ -64,7 +64,6 @@ SIPEndPoint::SIPEndPoint(OpalManager & mgr)
   , notifierTimeToLive(0, 0, 0, 1)   // 1 hour
   , natBindingTimeout(0, 0, 1)       // 1 minute
   , m_shuttingDown(false)
-  , m_dialogNotifyVersion(1)
   , m_defaultAppearanceCode(-1)
 
 #ifdef _MSC_VER
@@ -283,7 +282,7 @@ OpalTransport * SIPEndPoint::CreateTransport(const SIPURL & remoteURL, const PSt
     transport->AttachThread(PThread::Create(PCREATE_NOTIFIER(TransportThreadMain),
                                             (INT)transport,
                                             PThread::NoAutoDeleteThread,
-                                            PThread::NormalPriority,
+                                            PThread::HighestPriority,
                                             "SIP Transport"));
   return transport;
 }
@@ -352,9 +351,14 @@ PBoolean SIPEndPoint::MakeConnection(OpalCall & call,
   if (listeners.IsEmpty())
     return false;
 
-  PString callID;
-  SIPTransaction::GenerateCallID(callID);
-  return AddConnection(CreateConnection(call, callID, userData, TranslateENUM(remoteParty), NULL, NULL, options, stringOptions));
+  return AddConnection(CreateConnection(call, SIPURL::GenerateTag(), userData, TranslateENUM(remoteParty), NULL, NULL, options, stringOptions));
+}
+
+
+void SIPEndPoint::OnReleased(OpalConnection & connection)
+{
+  m_receivedConnectionTokens.RemoveAt(connection.GetIdentifier());
+  OpalEndPoint::OnReleased(connection);
 }
 
 
@@ -424,10 +428,7 @@ PBoolean SIPEndPoint::SetupTransfer(const PString & token,
   if (!callId.IsEmpty())
     options.SetAt(SIP_HEADER_REPLACES, callId);
 
-  PStringStream callID;
-  OpalGloballyUniqueID id;
-  callID << id << '@' << PIPSocket::GetHostName();
-  SIPConnection * connection = CreateConnection(call, callID, userData, TranslateENUM(remoteParty), NULL, NULL, 0, &options);
+  SIPConnection * connection = CreateConnection(call, SIPURL::GenerateTag(), userData, TranslateENUM(remoteParty), NULL, NULL, 0, &options);
   if (!AddConnection(connection))
     return false;
 
@@ -438,16 +439,11 @@ PBoolean SIPEndPoint::SetupTransfer(const PString & token,
 }
 
 
-PBoolean SIPEndPoint::ForwardConnection(SIPConnection & connection,  
-            const PString & forwardParty)
+PBoolean SIPEndPoint::ForwardConnection(SIPConnection & connection, const PString & forwardParty)
 {
   OpalCall & call = connection.GetCall();
   
-  PStringStream callID;
-  OpalGloballyUniqueID id;
-  callID << id << '@' << PIPSocket::GetHostName();
-
-  SIPConnection * conn = CreateConnection(call, callID, NULL, forwardParty, NULL, NULL);
+  SIPConnection * conn = CreateConnection(call, SIPURL::GenerateTag(), NULL, forwardParty, NULL, NULL);
   if (!AddConnection(conn))
     return PFalse;
 
@@ -464,14 +460,53 @@ PBoolean SIPEndPoint::OnReceivedPDU(OpalTransport & transport, SIP_PDU * pdu)
   if (PAssertNULL(pdu) == NULL)
     return PFalse;
 
+  const SIPMIMEInfo & mime = pdu->GetMIME();
+
+  /* Get tokens to determine the connection to operate on, not as easy as it
+     sounds due to allowing for talking to ones self, always thought madness
+     generally lies that way ... */
+
+  PString fromToken = mime.GetFieldParameter("from", "tag");
+  PString toToken = mime.GetFieldParameter("to", "tag");
+  bool hasFromConnection = HasConnection(fromToken);
+  bool hasToConnection = HasConnection(toToken);
+
+  PString token;
+
   // Adjust the Via list and send a trying in case it takes us a while to process request
   switch (pdu->GetMethod()) {
+    case SIP_PDU::Method_CANCEL :
+      token = m_receivedConnectionTokens(mime.GetCallID());
+      if (!token.IsEmpty()) {
+        threadPool.AddWork(new SIP_PDU_Work(*this, token, pdu));
+        return true;
+      }
+      break;
+
+    case SIP_PDU::Method_INVITE :
+      if (toToken.IsEmpty()) {
+        token = m_receivedConnectionTokens(mime.GetCallID());
+        if (!token.IsEmpty()) {
+          threadPool.AddWork(new SIP_PDU_Work(*this, token, pdu));
+          return true;
+        }
+
+        pdu->SendResponse(transport, SIP_PDU::Information_Trying, this);
+        return OnReceivedConnectionlessPDU(transport, pdu);
+      }
+
+      if (!hasToConnection) {
+        // Has to tag but doesn't correspond to anything, odd.
+        pdu->SendResponse(transport, SIP_PDU::Failure_TransactionDoesNotExist);
+        return false;
+      }
+      // Do next case
+
     default :
       pdu->SendResponse(transport, SIP_PDU::Information_Trying, this);
       // Do next case
 
     case SIP_PDU::Method_ACK :
-    case SIP_PDU::Method_CANCEL :
       pdu->AdjustVia(transport);
       // Do next case
 
@@ -479,24 +514,23 @@ PBoolean SIPEndPoint::OnReceivedPDU(OpalTransport & transport, SIP_PDU * pdu)
       break;
   }
 
-  // pass message off to thread pool
-  PString callID = pdu->GetMIME().GetCallID();
-  if (HasConnection(callID)) {
-    SIP_PDU_Work * work = new SIP_PDU_Work;
-    work->callID    = callID;
-    work->ep        = this;
-    work->pdu       = pdu;
-    threadPool.AddWork(work);
-    return true;
-  }
+  if (hasFromConnection && hasToConnection)
+    token = pdu->GetMethod() != SIP_PDU::NumMethods ? fromToken : toToken;
+  else if (hasFromConnection)
+    token = fromToken;
+  else if (hasToConnection)
+    token = toToken;
+  else
+    return OnReceivedConnectionlessPDU(transport, pdu);
 
-  return OnReceivedConnectionlessPDU(transport, pdu);
+  threadPool.AddWork(new SIP_PDU_Work(*this, token, pdu));
+  return true;
 }
 
 
 bool SIPEndPoint::OnReceivedConnectionlessPDU(OpalTransport & transport, SIP_PDU * pdu)
 {
-  if (pdu->GetMethod() == SIP_PDU::NumMethods) {
+  if (pdu->GetMethod() == SIP_PDU::NumMethods || pdu->GetMethod() == SIP_PDU::Method_CANCEL) {
     PSafePtr<SIPTransaction> transaction = GetTransaction(pdu->GetTransactionID(), PSafeReference);
     if (transaction != NULL)
       transaction->OnReceivedResponse(*pdu);
@@ -601,12 +635,10 @@ PBoolean SIPEndPoint::OnReceivedSUBSCRIBE(OpalTransport & transport, SIP_PDU & p
     return true;
 
   // Send initial NOTIFY as per spec 3.1.6.2/RFC3265
-  if (expires > 0 && eventPackage == SIPSubscribe::Dialog)
-    SendNotifyDialogInfo(SIPDialogNotification(handler->GetAddressOfRecord().AsString()));
-  else {
-    handler->SetBody(PString::Empty());
-    handler->SendRequest(expires > 0 ? SIPHandler::Subscribing : SIPHandler::Unsubscribing);
-  }
+  if (expires > 0)
+    handler->SendNotify(NULL);
+  else
+    handler->SendRequest(SIPHandler::Unsubscribing);
 
   return true;
 }
@@ -668,7 +700,7 @@ PBoolean SIPEndPoint::OnReceivedINVITE(OpalTransport & transport, SIP_PDU * requ
   // parse the incoming To field, and check if we accept incoming calls for this address
   SIPURL toAddr(mime.GetTo());
   if (!IsAcceptedAddress(toAddr)) {
-    PTRACE(2, "SIP\tIncoming INVITE from " << request->GetURI() << " for unknown address " << toAddr);
+    PTRACE(2, "SIP\tIncoming INVITE from " << request->GetURI() << " for unacceptable address " << toAddr);
     request->SendResponse(transport, SIP_PDU::Failure_NotFound, this);
     return PFalse;
   }
@@ -740,7 +772,7 @@ PBoolean SIPEndPoint::OnReceivedINVITE(OpalTransport & transport, SIP_PDU * requ
 
   // ask the endpoint for a connection
   SIPConnection *connection = CreateConnection(*call,
-                                               mime.GetCallID(),
+                                               SIPURL::GenerateTag(),
                                                NULL,
                                                request->GetURI(),
                                                newTransport,
@@ -751,12 +783,10 @@ PBoolean SIPEndPoint::OnReceivedINVITE(OpalTransport & transport, SIP_PDU * requ
     return PFalse;
   }
 
+  m_receivedConnectionTokens.SetAt(mime.GetCallID(), connection->GetToken());
+
   // Get the connection to handle the rest of the INVITE in the thread pool
-  SIP_PDU_Work * work = new SIP_PDU_Work;
-  work->ep        = this;
-  work->pdu       = request;
-  work->callID    = mime.GetCallID();
-  threadPool.AddWork(work);
+  threadPool.AddWork(new SIP_PDU_Work(*this, connection->GetToken(), request));
 
   return PTrue;
 }
@@ -1123,28 +1153,23 @@ bool SIPEndPoint::CanNotify(const PString & eventPackage)
 }
 
 
-bool SIPEndPoint::Notify(const SIPURL & aor, const PString & eventPackage, const PString & body)
+bool SIPEndPoint::Notify(const SIPURL & aor, const PString & eventPackage, const PObject & body)
 {
-  bool one = false;
+  bool atLeastOne = false;
 
   for (PSafePtr<SIPHandler> handler(activeSIPHandlers, PSafeReadWrite); handler != NULL; ++handler) {
     if (handler->GetMethod() == SIP_PDU::Method_NOTIFY &&
         handler->GetAddressOfRecord() == aor &&
-        handler->GetEventPackage() == eventPackage) {
-      handler->SetBody(body);
-      if (handler->SendRequest(SIPHandler::Subscribing))
-        one = true;
-    }
+        handler->GetEventPackage() == eventPackage &&
+        handler->SendNotify(&body))
+      atLeastOne = true;
   }
 
-  return one;
+  return atLeastOne;
 }
 
 PBoolean SIPEndPoint::Message(const PString & to, const PString & body)
 {
-  PString id;
-  SIPTransaction::GenerateCallID(id);
-
   // Create the SIPHandler structure
   PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByUrl(to, SIP_PDU::Method_MESSAGE, PSafeReadOnly);
 
@@ -1152,7 +1177,7 @@ PBoolean SIPEndPoint::Message(const PString & to, const PString & body)
   if (handler != NULL)
     handler->SetBody(body);
   else {
-    handler = new SIPMessageHandler(*this, to, body, "", id);
+    handler = new SIPMessageHandler(*this, to, body, "", SIPTransaction::GenerateCallID());
     activeSIPHandlers.Append(handler);
   }
 
@@ -1263,76 +1288,9 @@ void SIPEndPoint::OnDialogInfoReceived(const SIPDialogNotification & PTRACE_PARA
 }
 
 
-static void OutputParticipant(ostream & body, const char * name, const SIPDialogNotification::Participant & participant)
-{
-  if (participant.m_URI.IsEmpty())
-    return;
-
-  body << "    <" << name << ">\r\n";
-
-  if (!participant.m_identity.IsEmpty()) {
-    body << "      <identity";
-    if (!participant.m_display.IsEmpty())
-      body << " display\"" << participant.m_display << '"';
-    body << '>' << participant.m_identity << "</identity>\r\n";
-  }
-
-  body << "      <target uri=\"" << participant.m_URI << "\">\r\n";
-
-  if (participant.m_appearance >= 0)
-    body << "        <param pname=\"appearance\" pval=\"" << participant.m_appearance << "\"/>\r\n"
-            "        <param pname=\"x-line-id\" pval=\"" << participant.m_appearance << "\"/>\r\n";
-
-  if (participant.m_byeless)
-    body << "        <param pname=\"sip.byeless\" pval=\"true\"/>\r\n";
-
-  if (participant.m_rendering >= 0)
-    body << "        <param pname=\"sip.rendering\" pval=\"" << (participant.m_rendering > 0 ? "yes" : "no") << "\"/>\r\n";
-
-  body << "      </target>\r\n"
-       << "    </" << name << ">\r\n";
-}
-
-
 void SIPEndPoint::SendNotifyDialogInfo(const SIPDialogNotification & info)
 {
-  PStringStream body;
-  body << "<?xml version=\"1.0\"?>\r\n"
-          "<dialog-info xmlns=\"urn:ietf:params:xml:ns:dialog-info\" version=\""
-       << m_dialogNotifyVersion++ << "\" state=\"partial\" entity=\""
-       << info.m_entity << "\">\r\n";
-
-  if (!info.m_dialogId) {
-    // Start dialog XML tag
-    body << "  <dialog id=\"" << info.m_dialogId << '"';
-    if (!info.m_callId)
-      body << " call-id=\"" << info.m_callId << '"';
-    if (!info.m_local.m_dialogTag)
-      body << " local-tag=\"" << info.m_local.m_dialogTag << '"';
-    if (!info.m_remote.m_dialogTag)
-      body << " remote-tag=\"" << info.m_remote.m_dialogTag << '"';
-    body << " direction=\"" << (info.m_initiator ? "initiator" : "receiver") << "\">\r\n";
-
-    // State XML tag & value
-    body << "    <state";
-    if (info.m_eventType > SIPDialogNotification::NoEvent) {
-      body << " event=\"" << info.GetEventName() << '"';
-      if (info.m_eventCode > 0)
-        body << " code=\"" << info.m_eventCode << '"';
-    }
-    body << '>' << info.GetStateName() << "</state>\r\n";
-
-    // Participant XML tags (local/remopte)
-    OutputParticipant(body, "local", info.m_local);
-    OutputParticipant(body, "remote", info.m_remote);
-
-    // Close out dialog tag
-    body << "  </dialog>\r\n";
-  }
-
-  body << "</dialog-info>\r\n";
-
-  Notify(info.m_entity, SIPEventPackage(SIPSubscribe::Dialog), body);
+  Notify(info.m_entity, SIPEventPackage(SIPSubscribe::Dialog), info);
 }
 
 
@@ -1515,7 +1473,10 @@ void SIPEndPoint::OnRTPStatistics(const SIPConnection & connection,
 }
 
 SIPEndPoint::SIP_PDU_Thread::SIP_PDU_Thread(PThreadPoolBase & _pool)
-  : PThreadPoolWorkerBase(_pool) { }
+  : PThreadPoolWorkerBase(_pool)
+{
+  SetPriority(HighestPriority);
+}
 
 unsigned SIPEndPoint::SIP_PDU_Thread::GetWorkSize() const { return pduQueue.size(); }
 
@@ -1551,26 +1512,49 @@ void SIPEndPoint::SIP_PDU_Thread::Main()
     pduQueue.pop();
     mutex.Signal();
 
-    if (work->pdu->GetMethod() == SIP_PDU::NumMethods) {
-      PSafePtr<SIPTransaction> transaction = work->ep->GetTransaction(work->pdu->GetTransactionID(), PSafeReference);
-      if (transaction != NULL)
-        transaction->OnReceivedResponse(*work->pdu);
-      else {
-        PTRACE(3, "SIP\tCannot find transaction for response");
-      }
-    }
-    else if (!work->callID.IsEmpty()) {
-      PSafePtr<SIPConnection> connection = work->ep->GetSIPConnectionWithLock(work->callID, PSafeReference);
-      if (connection != NULL) 
-        connection->OnReceivedPDU(*work->pdu);
-    }
-
-    delete work->pdu;
+    work->OnReceivedPDU();
     delete work;
   }
 }
 
 
+SIPEndPoint::SIP_PDU_Work::SIP_PDU_Work(SIPEndPoint & ep, const PString & token, SIP_PDU * pdu)
+  : m_endpoint(ep)
+  , m_token(token)
+  , m_pdu(pdu)
+{
+}
+
+
+SIPEndPoint::SIP_PDU_Work::~SIP_PDU_Work()
+{
+  delete m_pdu;
+}
+
+
+void SIPEndPoint::SIP_PDU_Work::OnReceivedPDU()
+{
+  if (PAssertNULL(m_pdu) == NULL)
+    return;
+
+  if (m_pdu->GetMethod() == SIP_PDU::NumMethods) {
+    PSafePtr<SIPTransaction> transaction = m_endpoint.GetTransaction(m_pdu->GetTransactionID(), PSafeReference);
+    if (transaction != NULL)
+      transaction->OnReceivedResponse(*m_pdu);
+    else {
+      PTRACE(3, "SIP\tCannot find transaction for response");
+    }
+    return;
+  }
+
+  if (PAssert(!m_token.IsEmpty(), PInvalidParameter)) {
+    PSafePtr<SIPConnection> connection = m_endpoint.GetSIPConnectionWithLock(m_token, PSafeReference);
+    if (connection != NULL) 
+      connection->OnReceivedPDU(*m_pdu);
+  }
+}
+
+
 SIPEndPoint::InterfaceMonitor::InterfaceMonitor(SIPEndPoint & ep, PINDEX priority)
   : PInterfaceMonitorClient(priority) 
   , m_endpoint(ep)
