@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21841 $
+ * $Revision: 22097 $
  * $Author: rjongbloed $
- * $Date: 2008-12-19 04:59:52 +0000 (Fri, 19 Dec 2008) $
+ * $Date: 2009-02-21 04:55:10 +0000 (Sat, 21 Feb 2009) $
  */
 
 #include <ptlib.h>
@@ -314,7 +314,7 @@ PBoolean SIPURL::InternalParse(const char * cstr, const char * p_defaultScheme)
     if (startQuote == P_MAX_INDEX || endQuote == P_MAX_INDEX || startQuote >= endQuote) {
       // There are no double quotes around the display name, so take
       // everything before the start angle bracket
-      displayName = str.Left(startBracket-1).Trim();
+      displayName = str.Left(startBracket).Trim();
       }
     else {
       // Trim quotes off
@@ -499,6 +499,21 @@ PBoolean SIPURL::AdjustToDNS(PINDEX)
 #endif
 
 
+PString SIPURL::GenerateTag()
+{
+  return OpalGloballyUniqueID().AsString();
+}
+
+
+void SIPURL::SetTag(const PString & tag)
+{
+  if (fieldParameters.Find(";tag=") != P_MAX_INDEX)
+    return;
+
+  fieldParameters += ";tag=" + tag;
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
 
 SIPMIMEInfo::SIPMIMEInfo(PBoolean _compactForm)
@@ -1145,6 +1160,56 @@ void SIPMIMEInfo::SetRequire(const PString & v, bool overwrite)
 }
 
 
+void SIPMIMEInfo::GetAlertInfo(PString & info, int & appearance)
+{
+  info.MakeEmpty();
+  appearance = -1;
+
+  PString str = GetString("Alert-Info");
+  if (str.IsEmpty())
+    return;
+
+  PINDEX pos = str.Find('<');
+  PINDEX end = str.Find('>', pos);
+  if (pos == P_MAX_INDEX || end == P_MAX_INDEX) {
+    info = str;
+    return;
+  }
+
+  info = str(pos+1, end-1);
+
+  static const char appearance1[] = ";appearance=";
+  pos = str.Find(appearance1, end);
+  if (pos != P_MAX_INDEX) {
+    appearance = str.Mid(pos+sizeof(appearance1)).AsUnsigned();
+    return;
+  }
+
+  static const char appearance2[] = ";x-line-id";
+  pos = str.Find(appearance2, end);
+  if (pos != P_MAX_INDEX)
+    appearance = str.Mid(pos+sizeof(appearance2)).AsUnsigned();
+}
+
+
+void SIPMIMEInfo::SetAlertInfo(const PString & info, int appearance)
+{
+  if (appearance < 0) {
+    SetAt("Alert-Info", info);
+    return;
+  }
+
+  PStringStream str;
+  if (info[0] == '<')
+    str << info;
+  else
+    str << '<' << info << '>';
+  str << ";appearance=" << appearance;
+
+  SetAt("Alert-Info", str);
+}
+
+
 static bool LocateFieldParameter(const PString & fieldValue, const PString & paramName, PINDEX & start, PINDEX & end)
 {
   PINDEX semicolon = (PINDEX)-1;
@@ -1710,7 +1775,6 @@ void SIP_PDU::Construct(Methods meth,
   // construct Via:
   PINDEX dollar = via.Find('$');
 
-  OpalGloballyUniqueID branch;
   PStringStream str;
   str << "SIP/" << versionMajor << '.' << versionMinor << '/'
       << via.Left(dollar).ToUpper() << ' ';
@@ -1720,7 +1784,7 @@ void SIP_PDU::Construct(Methods meth,
     str << ip.AsString(true) << ':' << port;
   else
     str << via.Mid(dollar+1);
-  str << ";branch=z9hG4bK" << branch << ";rport";
+  str << ";branch=z9hG4bK" << OpalGloballyUniqueID() << ";rport";
 
   mime.SetVia(str);
 }
@@ -2227,16 +2291,10 @@ SDPSessionDescription * SIP_PDU::GetSDP()
 ////////////////////////////////////////////////////////////////////////////////////
 
 SIPDialogContext::SIPDialogContext()
-  : m_lastSentCSeq(0)
+  : m_callId(SIPTransaction::GenerateCallID())
+  , m_lastSentCSeq(0)
   , m_lastReceivedCSeq(0)
 {
-  GenerateCallID();
-}
-
-
-void SIPDialogContext::GenerateCallID()
-{
-  m_callId = PGloballyUniqueID().AsString() + '@' + PIPSocket::GetHostName();
 }
 
 
@@ -2256,7 +2314,7 @@ static void SetWithTag(const SIPURL & url, SIPURL & uri, PString & tag, bool gen
   }
 
   if (tag.IsEmpty() && generate)
-    tag = PGloballyUniqueID().AsString();
+    tag = SIPURL::GenerateTag();
 
   if (!tag.IsEmpty())
     uri.SetFieldParameters("tag="+tag);
@@ -2313,6 +2371,8 @@ void SIPDialogContext::Update(const SIP_PDU & pdu)
 {
   const SIPMIMEInfo & mime = pdu.GetMIME();
 
+  SetCallID(mime.GetCallID());
+
   if (m_routeSet.IsEmpty()) {
     // get the route set from the Record-Route response field according to 12.1.2
     // requests in a dialog do not modify the initial route set according to 12.2
@@ -2555,17 +2615,13 @@ PBoolean SIPTransaction::OnReceivedResponse(SIP_PDU & response)
 {
   // Stop the timers with asynchronous flag to avoid deadlock
   retryTimer.Stop(false);
-  completionTimer.Stop(false);
 
   PString cseq = response.GetMIME().GetCSeq();
 
-  // If is the response to a CANCEL we sent, then just ignore it
+  /* If is the response to a CANCEL we sent, then we stop retransmissions
+     and wait for the 487 Request Terminated to come in */
   if (cseq.Find(MethodNames[Method_CANCEL]) != P_MAX_INDEX) {
-    // Lock only if we have not already locked it in SIPInvite::OnReceivedResponse
-    if (LockReadWrite()) {
-      SetTerminated(Terminated_Cancelled);
-      UnlockReadWrite();
-    }
+    completionTimer = endpoint.GetPduCleanUpTimeout();
     return PFalse;
   }
 
@@ -2596,13 +2652,19 @@ PBoolean SIPTransaction::OnReceivedResponse(SIP_PDU & response)
 
       retry = 0;
       retryTimer = retryTimeoutMax;
-      completionTimer = endpoint.GetNonInviteTimeout();
+
+      int expiry = mime.GetExpires();
+      if (expiry > 0)
+        completionTimer.SetInterval(0, expiry);
+      else if (method == Method_INVITE)
+        completionTimer = endpoint.GetInviteTimeout();
+      else
+        completionTimer = endpoint.GetNonInviteTimeout();
     }
     else {
       PTRACE(3, "SIP\tTransaction " << cseq << " completed.");
       state = Completed;
       statusCode = response.GetStatusCode();
-      completionTimer = endpoint.GetPduCleanUpTimeout();
     }
 
     if (connection != NULL)
@@ -2666,8 +2728,35 @@ void SIPTransaction::OnTimeout(PTimer &, INT)
 {
   PSafeLockReadWrite lock(*this);
 
-  if (lock.IsLocked() && state <= Completed)
-    SetTerminated(state != Completed ? Terminated_Timeout : Terminated_Success);
+  if (lock.IsLocked()) {
+    switch (state) {
+      case Trying :
+        // Sent inityial command and got nothin'
+        SetTerminated(Terminated_Timeout);
+        break;
+
+      case Proceeding :
+        /* Got a 100 response and then nothing, give up with a CANCEL
+           just in case the other side is still there, and in particular
+           in the case of an INVITE where nobody answers */
+        Cancel();
+        break;
+
+      case Cancelling :
+        // We cancelled and finished waiting for retries
+        SetTerminated(Terminated_Cancelled);
+        break;
+
+      case Completed :
+        // We completed and finished waiting for retries
+        SetTerminated(Terminated_Success);
+        break;
+
+      default :
+        // Already terminated in some way
+        break;
+    }
+  }
 }
 
 
@@ -2688,7 +2777,11 @@ void SIPTransaction::SetTerminated(States newState)
     "Terminated_Aborted"
   };
 #endif
-  
+
+  // Terminated, so finished with timers
+  retryTimer.Stop(false);
+  completionTimer.Stop(false);
+
   if (state >= Terminated_Success) {
     PTRACE_IF(3, newState != Terminated_Success, "SIP\tTried to set state " << StateNames[newState] 
               << " for transaction " << mime.GetCSeq()
@@ -2702,19 +2795,29 @@ void SIPTransaction::SetTerminated(States newState)
   PTRACE(3, "SIP\tSet state " << StateNames[newState] << " for transaction " << mime.GetCSeq());
 
   // Transaction failed, tell the endpoint
-  switch (state) {
-    case Terminated_Success :
-      break;
+  if (state > Terminated_Success) {
+    switch (state) {
+      case Terminated_Timeout :
+      case Terminated_RetriesExceeded:
+        statusCode = SIP_PDU::Failure_RequestTimeout;
+        break;
 
-    case Terminated_Timeout :
-    case Terminated_RetriesExceeded:
-      statusCode = SIP_PDU::Failure_RequestTimeout;
-      // Do default case
+      case Terminated_TransportError :
+        statusCode = SIP_PDU::Local_TransportError;
+        break;
 
-    default :
-      endpoint.OnTransactionFailed(*this);
-      if (connection != NULL)
-        connection->OnTransactionFailed(*this);
+      case Terminated_Aborted :
+      case Terminated_Cancelled :
+        statusCode = SIP_PDU::Failure_RequestTerminated;
+        break;
+
+      default :// Prevent GNU warning
+        break;
+    }
+
+    endpoint.OnTransactionFailed(*this);
+    if (connection != NULL)
+      connection->OnTransactionFailed(*this);
   }
 
   if (oldState != Completed)
@@ -2722,9 +2825,9 @@ void SIPTransaction::SetTerminated(States newState)
 }
 
 
-void SIPTransaction::GenerateCallID(PString & callId)
+PString SIPTransaction::GenerateCallID()
 {
-  callId = PGloballyUniqueID().AsString() + '@' + PIPSocket::GetHostName();
+  return PGloballyUniqueID().AsString() + '@' + PIPSocket::GetHostName();
 }
 
 
@@ -2779,22 +2882,7 @@ PBoolean SIPInvite::OnReceivedResponse(SIP_PDU & response)
     }
   }
 
-  if (!SIPTransaction::OnReceivedResponse(response))
-    return false;
-
-  if (!LockReadWrite())
-    return false;
-
-  /* Handle response to outgoing call cancellation */
-  if (response.GetStatusCode() == Failure_RequestTerminated)
-    SetTerminated(Terminated_Success);
-
-  if (response.GetStatusCode()/100 == 1)
-    completionTimer = PTimeInterval(0, mime.GetExpires(180));
-    
-  UnlockReadWrite();
-
-  return true;
+  return SIPTransaction::OnReceivedResponse(response);
 }
 
 
@@ -2977,19 +3065,20 @@ SIPPublish::SIPPublish(SIPEndPoint & ep,
                        const PString & body)
   : SIPTransaction(ep, trans)
 {
-  SIPURL targetAddress = params.m_addressOfRecord;
-  PString addrStr = targetAddress.AsQuotedString();
+  SIPURL toAddress = params.m_addressOfRecord;
+  SIPURL fromAddress = toAddress;
+  fromAddress.SetTag();
 
   SIP_PDU::Construct(Method_PUBLISH,
-                     targetAddress,
-                     addrStr,
-                     addrStr+";tag="+OpalGloballyUniqueID().AsString(),
+                     toAddress,
+                     toAddress.AsQuotedString(),
+                     fromAddress.AsQuotedString(),
                      id,
                      endpoint.GetNextCSeq(),
                      ep.GetLocalURL(transport).GetHostAddress());
 
   mime.SetProductInfo(ep.GetUserAgent(), ep.GetProductInfo());
-  SIPURL contact = endpoint.GetLocalURL(trans, targetAddress.GetUserName());
+  SIPURL contact = endpoint.GetLocalURL(trans, toAddress.GetUserName());
   contact.Sanitise(SIPURL::ContactURI);
   mime.SetContact(contact);
   mime.SetExpires(params.m_expire);
@@ -3068,11 +3157,12 @@ SIPMessage::SIPMessage(SIPEndPoint & ep,
   : SIPTransaction(ep, trans)
 {
   SIPURL myAddress = endpoint.GetRegisteredPartyName(address.GetHostName(), transport);
+  myAddress.SetTag();
 
   SIP_PDU::Construct(Method_MESSAGE,
                      address.AsQuotedString(),
                      address.AsQuotedString(),
-                     myAddress.AsQuotedString()+";tag="+OpalGloballyUniqueID().AsString(),
+                     myAddress.AsQuotedString(),
                      id,
                      endpoint.GetNextCSeq(),
                      ep.GetLocalURL(transport).GetHostAddress());
@@ -3094,7 +3184,7 @@ SIPPing::SIPPing(SIPEndPoint & ep,
                      address.AsQuotedString(),
                      address.AsQuotedString(),
                      "sip:"+address.GetUserName()+"@"+address.GetHostName(),
-                     OpalGloballyUniqueID().AsString() + "@" + PIPSocket::GetHostName(),
+                     GenerateCallID(),
                      endpoint.GetNextCSeq(),
                      ep.GetLocalURL(transport).GetHostAddress());
   mime.SetContentType("text/plain;charset=UTF-8");
@@ -3145,12 +3235,13 @@ SIPOptions::SIPOptions(SIPEndPoint & ep,
 {
   // Build the correct From field
   SIPURL myAddress = endpoint.GetRegisteredPartyName(address.GetHostName(), transport);
+  myAddress.SetTag();
 
   SIP_PDU::Construct(Method_OPTIONS,
                      address,
                      address.AsQuotedString(),
-                     myAddress.AsQuotedString()+";tag="+OpalGloballyUniqueID().AsString(),
-                     OpalGloballyUniqueID().AsString() + "@" + PIPSocket::GetHostName(),
+                     myAddress.AsQuotedString(),
+                     GenerateCallID(),
                      endpoint.GetNextCSeq(),
                      ep.GetLocalURL(transport).GetHostAddress());
   mime.SetAccept("application/sdp, application/media_control+xml, application/dtmf, application/dtmf-relay");
